# -*- python -*-
#
# Hippotat - Asinine IP Over HTTP program
# hippotatlib/slip.py - SLIP handling
#
# Copyright 2017 Ian Jackson
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version, with the "CAF Login
# Exception" as published by Ian Jackson (version 2, or at your option
# any later version) as an Additional Permission.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License and the CAF Login Exception along with this program, in the
# file AGPLv3+CAFv2.  If not, email Ian Jackson
# <ijackson@chiark.greenend.org.uk>.


end = b'\300'
esc = b'\333'
esc_end = b'\334'
esc_esc = b'\335'
delimiter = end

def encode(packet):
  return (packet
          .replace(esc, esc + esc_esc)
          .replace(end, esc + esc_end))

def decode(data, keep_tail=False):
  #print('DECODE ', repr(data))
  out = []
  inp = data.split(end)
  tail = []
  if keep_tail:
    tail.append(inp.pop())
  for packet in inp:
    pdata = b''
    while True:
      eix = packet.find(esc)
      if eix == -1:
        pdata += packet
        break
      #print('ESC ', repr((pdata, packet, eix)))
      pdata += packet[0 : eix]
      ck = packet[eix+1]
      #print('ESC... %o' % ck)
      if   ck == esc_esc[0]: pdata += esc
      elif ck == esc_end[0]: pdata += end
      else: raise ValueError('invalid SLIP escape 0%o %#x' % (ck, ck))
      packet = packet[eix+2 : ]
    out.append(pdata)
  #print('DECODED ', repr(out))
  out += tail
  return out
# -*- python -*-

